﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*******************************************************************************
#**  This file is part of HgWeb Manager.                                     ***
#**                                                                          ***
#**  Copyright (C) 2014                                                      ***
#**  CodeGNU Solutions <Licensing _AT_ CodeGNU _DOT_ com>                    ***
#**                                                                          ***
#**  This program is free software: you can redistribute it and/or modify it ***
#**  under the terms of the GNU Affero General Public License as published   ***
#**  by the Free Software Foundation, either version 3 of the License, or    ***
#**  (at your option) any later version.                                     ***
#**                                                                          ***
#**  This program is distributed in the hope that it will be useful, but     ***
#**  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
#**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
#**  See the GNU Affero General Public License for more details.             ***
#**                                                                          ***
#**  You should have received a copy of the GNU Affero General Public License***
#**  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
#*******************************************************************************

"""
 IMPORTS
"""
import repository


def create(settings, repository):
	import subprocess
	import sys
	import os
	import stat
	import shutil
	print "Generating repository:" + repository.StorageName
	repoBase = settings.RepositoryBase + os.sep + repository.StorageName
	config = repoBase + os.sep + ".hg" + os.sep + "hgrc"

	# Create the requested repository.
	try:
		repoCreateStatus = subprocess.call([settings.HgCommand, "init", repoBase])

		if 0 == repoCreateStatus:
			print "Created repository directories."
		else:
			print >>sys.stderr, "Failed to create repository directories.", repoCreateStatus
			return False
	except OSError as e:
		print >>sys.stderr, "Initialisation error({0}): {1}".format(e.errno, e.strerror)
		return False

	# Fill out the details of about the repository.
	with open(config, "w") as hgrc:
		hgrc.write("[web]\n")
		if repository.DisplayName is None:
			hgrc.write("name = " + repository.StorageName + '\n')
		else:
			hgrc.write("name = " + repository.DisplayName + '\n')
		if repository.Description is not None:
			hgrc.write("description = " + repository.Description + '\n')
		if repository.Contact is not None:
			hgrc.write("contact = " + repository.Contact + '\n')
	print "Wrote repository details."

	# Ensure the repository details isn't accidentally modified.
	hgrcMode = os.stat(config)[stat.ST_MODE]
	os.chmod(config, hgrcMode & ~stat.S_IWUSR & ~stat.S_IWGRP & ~stat.S_IWOTH)
	print "Write protected repository details."

	# Ensure the repository store isn't accidentally deleted, moved, or blocked in Windows.
	if "win32" == sys.platform:
		repoStoreStatus = subprocess.call(["attrib.exe", "+H +I +S " + repoBase + os.sep + ".hg"])
		if 0 == repoStoreStatus:
			print "Secured repository store."
		else:
			print >>sys.stderr, "Failed to secure repository store.", repoStoreStatus
			exit(repoStoreStatus)

	# Add the new repository to the web register.
	with open(settings.HgWebPath + os.sep + "hgweb.config", "a") as hgweb:
		hgweb.write('\n' + repository.StorageName + " = " + repoBase)
	return True
